function [sys_out,blk_out]=ltp2lti(fun,T,nt,method)
% LTP2LTI  -  Lifting procedure for mu-analysis of linear time-periodic
% systems
%--------------------------------------------------------------------------
% PURPOSE
% This routine computes a continuous-time LTI system associated with the
% continous-time periodic system (LTP) using the lifting procedure: 
%    * the period T is sampled into N sub-intervals 
%    * the system is assumed to be LTI in the sub-interval i and discretize
%      according to a prescribed method ==> SYSD(i)
%    * the N discrete-time system SYSD(i) are integrated over one period T,
%    * the final discrete-time system (sampling period = T) is converted
%      to a continuous-time system using TUSTIN tranformation.
%--------------------------------------------------------------------------
% SYNOPSIS
% [SS_LIFT,BLK_LIFT]=ltp2lti(FUN,T,N) or
% [SS_LIFT,BLK_LIFT]=ltp2lti(FUN,T,N,'method')
%  
% INPUT ARGUMENTS
%    FUN  : function to compute the continuous time-periodic M-DELTA system
%           at time TIME according to the synthax :
%           [SS_M,BLK]=FUN(T,TIME) where SS_M is an SS_object describing
%            the interconnection structure M (at time TIME) and BLK the
%            matrix defining the structure of the uncertainty block DELTA.
%     T   : time-period (s)
%     N   : number of sub-intervals in the period (integer)
% 'method':  optional argument:
%           * 'zoh' assumes a zero order hold on the inputs.
%           * 'foh' assumes a first order hold on the inputs (default).
%           * 'tustin' Convert to discrete time using the bilinear 
%              (Tustin) approximation to the derivative.
%  
% OUTPUT ARGUMENTS
%  SS_LIFT  : augmented (lifted) LTI model
%  BLK_LIFT : augmented uncertainty desciption matrix
%
% REMARK #1:
% Output arguments (SS_LIFT and BLK_LIFT) of function LTP2LTI and 
% output arguments (SS_M and BLK) of  function FUN are in the standard
% of the Skew Mu Toolbox (SMTv3).
%
% REMARK #2:
% When output arguments are directly used in MU_MARGIN :
% [MUUB,WC,BNDS,TAB] = mu_margin(SS_LIFT,BLK_LIFT)
% the actual critical pulsation W is linked to WC by the relation:
%              W=2/T*ATAN(T*WC/2)
%
% See also: mu_margin

% Leonardo Sanches, Daniel ALazard  04/2011


if nargin==3,
      method='foh';
end
t_vec=linspace(0,T,nt+1);
hh=t_vec(2)-t_vec(1);
for it=1:nt 
    t=t_vec(it);
    eval(['sysi{',int2str(it),'}=',fun,'(',num2str(T),',',num2str(t),');']);
    sysdis{it}=c2d(sysi{it},hh,method);
end

%% Applying the Lifting Procedure

%Computing successive matrix products  M(k,j)=Ad(k-1)*Ad(k-2)*...*Ad(j)
%with Ad(i)=sysdis{i-1}.a  and j <=k ; k>=1 in a lower triangular cell
%array.
for k=1:nt,
    M{k,k}=sysdis{k}.a;
    for j=1:nt-k,
      M{k+j,k}=sysdis{k+j}.a*M{k+j-1,k};
    end
end    


A = M{nt,1};

B=[];
for ii=2:nt;
    B=[B M{nt,ii}*sysdis{ii-1}.b];
end
B=[B sysdis{nt}.b];

C=sysdis{1}.c;
for ii=1:nt-1,
    C=[C;sysdis{ii+1}.c*M{ii,1}];
end

D=[];
for ii=1:nt D=daug(D,sysdis{ii}.d);end,
[n_out,n_in]=size(sysdis{1}.d);
for k=2:nt,
    D((k-1)*n_out+1:k*n_out,(k-2)*n_in+1:(k-1)*n_in)=sysdis{k}.c*sysdis{k-1}.b;
    for j=1:nt-k,
        D((k+j-1)*n_out+1:(k+j)*n_out,(k-2)*n_in+1:(k-1)*n_in)=sysdis{k+j}.c*M{k+j-1,k}*sysdis{k-1}.b;
    end
end

% The Matrix V  and W transform the organized one to a desorganized
V=zeros(size(D,2));
for ielem=1:nt
    for indDelt=1:n_in
        V((ielem-1)*n_in+indDelt,(indDelt-1)*nt+ielem)=1;
    end
end         
W=zeros(size(D,1));
for ielem=1:nt
    for indDelt=1:n_out
        W((ielem-1)*n_out+indDelt,(indDelt-1)*nt+ielem)=1;
    end
end         

%% Arranging the vector related to Delta Matrix
sys_out=ss(A,B*V,W'*C,W'*D*V,T);
% Back to continous time:
sys_out=d2c(sys_out,'tustin');
eval(['[bid,blk]=',fun,'(',num2str(T),',',num2str(0),');']);
blk_out=nt*blk;
